import {Type} from './Type.js'
import {Implementations, isPlainObject} from './helpers.js'

export class TypeDispatcher {
   constructor(...specs) {
      for (const spec of specs) this.merge(spec)
   }

   merge(spec) {
      if (!spec) return
      if (typeof spec != 'object') {
         throw new TypeError(
            `TypeDispatcher specifications must be objects, not '${spec}'.`)
      }
      for (const key in spec) {
         const val = spec[key]
         if (val instanceof Type) {
            console.log(`Pretending to install type ${key}: ${val}`)
            continue
         }
         if (val instanceof Implementations) {
            console.log(`Pretending to install implementations for ${key}`)
            console.log(`  --> ${val}`)
            continue
         }
         if (typeof val === 'function') {
            console.log(`Pretend install of catchall implementation for ${key}`)
            continue
         }
         if (isPlainObject(val)) {
            this.merge(val)
            continue
         }
         console.log(`Pretend install of catchall value for ${key}: ${val}`)
      }
   }
}
